@@ -9,7 +9,6 @@ log = logging.getLogger(__name__)
 from . import stream
 from . import dsp
 from . import sampling
-from . import train
 from . import common
 from . import framing
 from . import equalizer
@@ -20,19 +19,20 @@ class Receiver(object):
 
     def __init__(self, config, pylab=None):
         self.stats = {}
-        self.plt = pylab or common.Dummy()
+        self.plt = pylab
         self.modem = dsp.MODEM(config.symbols)
         self.frequencies = np.array(config.frequencies)
         self.omegas = 2 * np.pi * self.frequencies / config.Fs
         self.Nsym = config.Nsym
         self.Tsym = config.Tsym
-        self.iters_per_report = config.baud  # report once per second
+        self.iters_per_update = 100  # [ms]
         self.modem_bitrate = config.modem_bps
         self.equalizer = equalizer.Equalizer(config)
         self.carrier_index = config.carrier_index
+        self.output_size = 0  # number of bytes written to output stream
 
-    def _prefix(self, symbols, gain=1.0, skip=5):
-        S = common.take(symbols, len(train.prefix))
+    def _prefix(self, symbols, gain=1.0):
+        S = common.take(symbols, len(equalizer.prefix))
         S = S[:, self.carrier_index] * gain
         sliced = np.round(np.abs(S))
         self.plt.figure()
@@ -42,40 +42,23 @@ class Receiver(object):
         bits = np.array(sliced, dtype=int)
         self.plt.subplot(122)
         self.plt.plot(np.abs(S))
-        self.plt.plot(train.prefix)
-        if any(bits != train.prefix):
+        self.plt.plot(equalizer.prefix)
+        if any(bits != equalizer.prefix):
             raise ValueError('Incorrect prefix')
-
         log.debug('Prefix OK')
 
-        nonzeros = np.array(train.prefix, dtype=bool)
-        pilot_tone = S[nonzeros]
-        phase = np.unwrap(np.angle(pilot_tone)) / (2 * np.pi)
-        indices = np.arange(len(phase))
-        a, b = dsp.linear_regression(indices[skip:-skip], phase[skip:-skip])
-        self.plt.figure()
-        self.plt.plot(indices, phase, ':')
-        self.plt.plot(indices, a * indices + b)
-
-        freq_err = a / (self.Tsym * self.frequencies[self.carrier_index])
-        last_phase = a * indices[-1] + b
-        log.debug('Current phase on carrier: %.3f', last_phase)
-
-        log.debug('Frequency error: %.2f ppm', freq_err * 1e6)
-        self.plt.title('Frequency drift: {0:.3f} ppm'.format(freq_err * 1e6))
-        return freq_err
-
     def _train(self, sampler, order, lookahead):
         Nfreq = len(self.frequencies)
-        train_symbols = self.equalizer.train_symbols(train.equalizer_length)
+        equalizer_length = equalizer.equalizer_length
+        train_symbols = self.equalizer.train_symbols(equalizer_length)
         train_signal = self.equalizer.modulator(train_symbols) * Nfreq
 
-        prefix = postfix = train.silence_length * self.Nsym
-        signal_length = train.equalizer_length * self.Nsym + prefix + postfix
+        prefix = postfix = equalizer.silence_length * self.Nsym
+        signal_length = equalizer_length * self.Nsym + prefix + postfix
 
         signal = sampler.take(signal_length + lookahead)
 
-        coeffs = self.equalizer.equalize_signal(
+        coeffs = equalizer.train(
             signal=signal[prefix:-postfix],
             expected=train_signal,
             order=order, lookahead=lookahead
@@ -87,8 +70,12 @@ class Receiver(object):
         equalization_filter = dsp.FIR(h=coeffs)
         equalized = list(equalization_filter(signal))
         equalized = equalized[prefix+lookahead:-postfix+lookahead]
+        self._verify_training(equalized, train_symbols)
+        return equalization_filter
 
-        symbols = self.equalizer.demodulator(equalized, train.equalizer_length)
+    def _verify_training(self, equalized, train_symbols):
+        equalizer_length = equalizer.equalizer_length
+        symbols = self.equalizer.demodulator(equalized, equalizer_length)
         sliced = np.array(symbols).round()
         errors = np.array(sliced - train_symbols, dtype=np.bool)
         error_rate = errors.sum() / errors.size
@@ -107,8 +94,6 @@ class Receiver(object):
                                 '$F_c = {0} Hz$'.format(freq), index=i)
         assert error_rate == 0, error_rate
 
-        return equalization_filter
-
     def _demodulate(self, sampler, symbols):
         streams = []
         symbol_list = []
@@ -137,40 +122,39 @@ class Receiver(object):
                 self.stats['rx_bits'] = self.stats['rx_bits'] + len(bits)
                 yield bits
 
-            if i % self.iters_per_report == 0:
-                err = np.array([e for v in errors.values() for e in v])
-                err = np.mean(np.angle(err))/(2*np.pi) if len(err) else 0
-                errors.clear()
+            if i % self.iters_per_update == 0:
+                self._update_sampler(i, errors, sampler)
 
-                duration = time.time() - self.stats['rx_start']
-                sampler.freq -= 0.01 * err * self.Tsym
-                sampler.offset -= err
-                log.debug(
-                    'Got  %8.1f kB, realtime: %6.2f%%, drift: %+5.2f ppm',
-                    self.stats['rx_bits'] / 8e3,
-                    duration * 100.0 / (i*self.Tsym),
-                    (1.0 - sampler.freq) * 1e6
-                )
+    def _update_sampler(self, iter_index, errors, sampler):
+        err = np.array([e for v in errors.values() for e in v])
+        err = np.mean(np.angle(err))/(2*np.pi) if len(err) else 0
+        errors.clear()
 
-    def start(self, signal, gain=1.0):
-        sampler = sampling.Sampler(signal, sampling.Interpolator())
+        duration = time.time() - self.stats['rx_start']
+        sampler.freq -= 0.01 * err * self.Tsym
+        sampler.offset -= err
+        log.debug(
+            'Got  %10.3f kB, realtime: %6.2f%%, drift: %+5.2f ppm',
+            self.stats['rx_bits'] / 8e3,
+            duration * 100.0 / (iter_index * self.Tsym),
+            (1.0 - sampler.freq) * 1e6
+        )
+
+    def run(self, sampler, gain, output):
         symbols = dsp.Demux(sampler, omegas=self.omegas, Nsym=self.Nsym)
-        freq_err = self._prefix(symbols, gain=gain)
-        sampler.freq -= freq_err
+        self._prefix(symbols, gain=gain)
 
-        filt = self._train(sampler, order=11, lookahead=6)
+        filt = self._train(sampler, order=20, lookahead=20)
         sampler.equalizer = lambda x: list(filt(x))
 
         bitstream = self._demodulate(sampler, symbols)
-        self.bitstream = itertools.chain.from_iterable(bitstream)
+        bitstream = itertools.chain.from_iterable(bitstream)
 
-    def run(self, output):
-        data = framing.decode(self.bitstream)
-        self.size = 0
+        data = framing.decode(bitstream)
         for chunk in common.iterate(data=data, size=256,
                                     truncate=False, func=bytearray):
             output.write(chunk)
-            self.size += len(chunk)
+            self.output_size += len(chunk)
 
     def report(self):
         if self.stats:
@@ -181,7 +165,8 @@ class Receiver(object):
                       100 * duration / audio_time if audio_time else 0)
 
             log.info('Received %.3f kB @ %.3f seconds = %.3f kB/s',
-                     self.size * 1e-3, duration, self.size * 1e-3 / duration)
+                     self.output_size * 1e-3, duration,
+                     self.output_size * 1e-3 / duration)
 
             self.plt.figure()
             symbol_list = np.array(self.stats['symbol_list'])
@@ -209,25 +194,45 @@ class Receiver(object):
         self.plt.title(title)
 
 
-def main(config, src, dst, pylab=None):
+class Dumper(object):
+    def __init__(self, src, dst):
+        self.src = src
+        self.dst = dst
+
+    def read(self, size):
+        data = self.src.read(size)
+        self.dst.write(data)
+        return data
+
+
+def main(config, src, dst, dump_audio=None, pylab=None):
+    if dump_audio:
+        src = Dumper(src, dump_audio)
     reader = stream.Reader(src, data_type=common.loads)
     signal = itertools.chain.from_iterable(reader)
 
-    to_skip = int(config.skip_start * config.Fs)
     log.debug('Skipping %.3f seconds', config.skip_start)
-    common.take(signal, to_skip)
+    common.take(signal, int(config.skip_start * config.Fs))
 
-    detector = detect.Detector(config=config)
+    pylab = pylab or common.Dummy()
+    detector = detect.Detector(config=config, pylab=pylab)
     receiver = Receiver(config=config, pylab=pylab)
-    success = False
     try:
         log.info('Waiting for carrier tone: %.1f kHz', config.Fc / 1e3)
-        signal, amplitude = detector.run(signal)
-        receiver.start(signal, gain=1.0/amplitude)
-        receiver.run(dst)
-        success = True
+        signal, amplitude, freq_error = detector.run(signal)
+
+        freq = 1 / (1.0 + freq_error)  # receiver's compensated frequency
+        log.debug('Frequency correction: %.3f ppm', (freq - 1) * 1e6)
+
+        gain = 1.0 / amplitude
+        log.debug('Gain correction: %.3f', gain)
+
+        sampler = sampling.Sampler(signal, sampling.Interpolator(), freq=freq)
+        receiver.run(sampler, gain=1.0/amplitude, output=dst)
+        return True
     except Exception:
         log.exception('Decoding failed')
-
-    receiver.report()
-    return success
+        return False
+    finally:
+        dst.flush()
+        receiver.report()
